package com.yesheng.util;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.yesheng.anno.RequiresPermissionsDesc;
import com.yesheng.dto.PermissionDto;
import com.yesheng.dto.PermissionTreeDto;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.reflect.MethodUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Controller;
import org.springframework.util.ClassUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class PermissionUtil {


    private final static String basePackage = "com.yesheng.web";

    public static List<PermissionTreeDto> getPermissionTree(List<PermissionDto> permissionDtos) {
        List<PermissionTreeDto> tree = new ArrayList<>();
        for (PermissionDto permissionDto : permissionDtos) {
            RequiresPermissionsDesc permissionsDesc = permissionDto.getRequiresPermissionsDesc();
            String[] menus = permissionsDesc.menu();
            PermissionTreeDto root = new PermissionTreeDto();
            //                 menu  {"系统管理", "角色管理"}, value = "角色查询" , menu是一个数组结构，第一项是一级菜单，第二项是二级菜单，value挂在二级菜单下
            if (menus.length != 2) {
                throw new RuntimeException("目前只支持两级菜单");
            }
            String menu1 = menus[0];
            if (tree.stream().anyMatch(a -> a.getName().equals(menu1))) {
                root = tree.stream().filter(a -> a.getName().equals(menu1)).findFirst().orElse(null);
                if (null == root) {
                    root = new PermissionTreeDto();
                    root.setName(menu1);
                    tree.add(root);
                }
            } else {
                root.setName(menu1);
                tree.add(root);
            }
            PermissionTreeDto child = new PermissionTreeDto();
            //检查二级菜单是否已存在，存在则直接挂在二级菜单下，不存在则创建
            String menu2 = menus[1];
            if (root.getChildren() != null && root.getChildren().stream().anyMatch(a -> a.getName().equals(menu2))) {
                child = root.getChildren().stream().filter(a -> a.getName().equals(menu2)).findFirst().orElse(null);
                if (null == child) {
                    child = new PermissionTreeDto();
                    child.setName(menu2);
                    root.getChildren().add(child);
                }
            } else {
                child.setName(menu2);
                if (root.getChildren() == null) {
                    root.setChildren(new ArrayList<>());
                }
                root.getChildren().add(child);
            }
            // value挂在二级菜单下
            PermissionTreeDto leaf = new PermissionTreeDto();
            leaf.setName(permissionDto.getApi());
            leaf.setDesc(permissionsDesc.value());
            child.setChildren(List.of(leaf));
        }
        return tree;
    }


    public static List<PermissionDto> getPermissionList() {
        ApplicationContext context = SpringContextUtil.getApplicationContext();
        Map<String, Object> map = context.getBeansWithAnnotation(Controller.class);
        List<PermissionDto> permissionDtos = new ArrayList<>();
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            Object value = entry.getValue();
            if (!StringUtils.contains(ClassUtils.getPackageName(value.getClass()), basePackage)) {
                continue;
            }
            Class<?> clz = value.getClass();
            RequestMapping clazzRequestMapping = AnnotationUtils.findAnnotation(clz, RequestMapping.class);
            String prefix = "";

            List<Method> methods = MethodUtils.getMethodsListWithAnnotation(clz, SaCheckPermission.class);
            for (Method method : methods) {
                SaCheckPermission saCheckPermission = AnnotationUtils.findAnnotation(method, SaCheckPermission.class);
                if (saCheckPermission == null) {
                    continue;
                }
                PermissionDto permissionDto = new PermissionDto();
                permissionDto.setSaCheckPermission(saCheckPermission);
                String s = saCheckPermission.value()[0];
                if (clazzRequestMapping != null) {
                    prefix = clazzRequestMapping.value()[0] + ":";
                }
                permissionDto.setApi(prefix + s);
                permissionDto.setRequiresPermissionsDesc(AnnotationUtils.findAnnotation(method, RequiresPermissionsDesc.class));
                permissionDtos.add(permissionDto);
            }
        }
        return permissionDtos;
    }

}
