package com.yesheng.service.impl;

import com.yesheng.db.entity.SysRolePermission;
import com.yesheng.db.entity.SysRoleUser;
import com.yesheng.db.mapper.SysRoleMapper;
import com.yesheng.db.mapper.SysRolePermissionMapper;
import com.yesheng.db.mapper.SysRoleUserMapper;
import com.yesheng.service.RoleService;
import com.yesheng.service.dto.RoleMapDto;
import com.yesheng.service.dto.UserRoleDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 处理用户角色、权限相关的业务逻辑
 */
@Service
public class RoleServiceImpl implements RoleService {


    private final SysRoleUserMapper roleUserMapper;

    private final SysRolePermissionMapper rolePermissionMapper;

    private final SysRoleMapper roleMapper;

    @Autowired
    public RoleServiceImpl(SysRoleUserMapper roleUserMapper, SysRolePermissionMapper rolePermissionMapper, SysRoleMapper roleMapper) {
        this.roleMapper = roleMapper;
        this.roleUserMapper = roleUserMapper;
        this.rolePermissionMapper = rolePermissionMapper;
    }

    //根据用户id 查询用户角色
    @Override
    public UserRoleDto getUserRoleByUserId(String userId) {
        Integer userIdInt = Integer.parseInt(userId);
        List<SysRoleUser> roleUsers = roleUserMapper.selectByUserId(userIdInt);
        if (null == roleUsers || roleUsers.isEmpty()) {
            return null;
        }
        UserRoleDto userRoleDto = new UserRoleDto();
        userRoleDto.setUserId(userIdInt);
        Set<Integer> roleIdSet = roleUsers.stream().map(SysRoleUser::getRoleId).collect(Collectors.toSet());
        List<SysRolePermission> rolePermissions = rolePermissionMapper.selectByRoleIds(roleIdSet);
        userRoleDto.setRoleMapDtoList(List.of());
        roleMapper.selectByIds(roleIdSet).forEach(role -> {
            RoleMapDto roleMapDto = new RoleMapDto();
            roleMapDto.setRoleName(role.getName());
            roleMapDto.setRoleId(role.getId());
            roleMapDto.setRoleDesc(role.getDescribeStr());
            roleMapDto.setPemissionList(rolePermissions.stream().filter(rolePermission -> rolePermission.getRoleId()
                    .equals(role.getId())).map(SysRolePermission::getPermission).collect(Collectors.toList()));
            List<RoleMapDto> roleMapDtoList = new ArrayList<>();
            roleMapDtoList.add(roleMapDto);
            userRoleDto.setRoleMapDtoList(roleMapDtoList);
        });

        return userRoleDto;
    }


    @Override
    public List<String> getPermissionList(Object loginId, String loginType) {
        UserRoleDto userRoleDto = getUserRoleByUserId((String) loginId);
        if (null == userRoleDto) {
            return List.of();
        } else {
            return userRoleDto.getRoleMapDtoList().stream().flatMap(roleMapDto -> roleMapDto.getPemissionList().stream()).distinct().collect(Collectors.toList());
        }
    }

    @Override
    public List<String> getRoleList(Object loginId, String loginType) {
        UserRoleDto userRoleByUserId = getUserRoleByUserId((String) loginId);
        if (null == userRoleByUserId) {
            return List.of();
        } else {
            return userRoleByUserId.getRoleMapDtoList().stream().map(RoleMapDto::getRoleName).collect(Collectors.toList());
        }
    }
}
